'use strict';

require('path');
require('hable');
require('signal-exit');
const __chunk_1 = require('./tib-chunk.js');
require('os');
require('child_process');
require('tree-kill');
require('vue-template-compiler');
require('@babel/core');
require('@babel/parser');
const __chunk_2 = require('./tib-chunk2.js');
const __chunk_3 = require('./tib-chunk3.js');
require('fs');
const __chunk_5 = require('./tib-chunk5.js');

class PuppeteerWebpage extends __chunk_3.Webpage {
  async open(url, readyCondition = 'body') {
    this.page = await this.driver.newPage();

    await this.browser.callHook('page:created', this.page);

    await this.page.goto(url);

    if (readyCondition) {
      await this.page.waitFor(readyCondition);
    }

    return this.returnProxy()
  }

  runScript(fn, ...args) {
    return this.page.evaluate(fn, ...args)
  }

  getHtml() {
    /* istanbul ignore next */
    const pageFn = () => window.document.documentElement.outerHTML;
    return this.page.evaluate(pageFn)
  }

  getTitle() {
    return this.page.title()
  }
}

class PuppeteerCoreBrowser extends __chunk_2.Browser {
  async _loadDependencies() {
    super._loadDependencies();

    if (!PuppeteerCoreBrowser.core) {
      PuppeteerCoreBrowser.core = await this.loadDependency('puppeteer-core');
    }
  }

  setHeadless() {
    super.setHeadless();
    this.config.browserArguments.push('--headless');
  }

  setLogLevel(types) {
    const typeMap = {
      warning: 'warn'
    };

    if (types && typeof types === 'string') {
      types = [types];
    }

    this.hook('page:created', (page) => {
      /* eslint-disable no-console */
      page.on('console', (msg) => {
        const msgType = msg.type();
        let type = typeMap[msgType] || msgType;
        if (!types || types.includes(msgType) || types.includes(type)) {
          if (!console[type]) {
            console.warn(`Unknown console type ${type}`);
            type = 'log';
          }
          console[type](msg.text());
        }
      });
      /* eslint-enable no-console */
    });
  }

  async _start(capabilities, ...args) {
    // https://github.com/GoogleChrome/puppeteer/blob/master/docs/api.md#puppeteerlaunchoptions
    const launchOptions = {
      args: [
        '--no-sandbox',
        '--disable-setuid-sandbox',
        ...this.config.browserArguments,
        ...args
      ],
      ...capabilities
    };

    if (this.constructor.name === 'PuppeteerCoreBrowser') {
      let executablePath = process.env.PUPPETEER_EXECUTABLE_PATH;
      if (!executablePath) {
        const detector = new __chunk_5.ChromeDetector();
        executablePath = detector.detect();
      }

      if (!executablePath) {
        throw new __chunk_1.BrowserError(this, `Could not find a Chrome executable`)
      }

      launchOptions.executablePath = executablePath;
    }

    this.driver = await PuppeteerCoreBrowser.core.launch(launchOptions);
  }

  async _close() {
    if (!this.driver) {
      return
    }

    await this.driver.close();
  }

  _page(url, readyCondition) {
    const page = new PuppeteerWebpage(this);
    return page.open(url, readyCondition)
  }
}

exports.default = PuppeteerCoreBrowser;
