'use strict';

require('path');
require('hable');
require('signal-exit');
const __chunk_1 = require('./tib-chunk.js');
require('os');
require('child_process');
require('tree-kill');
require('vue-template-compiler');
require('@babel/core');
require('@babel/parser');
const __chunk_2 = require('./tib-chunk2.js');
const __chunk_3 = require('./tib-chunk3.js');

class SeleniumWebpage extends __chunk_3.Webpage {
  async open(url, readyCondition = 'body') {
    await this.driver.get(url);

    await this.browser.callHook('page:created', this.driver);

    SeleniumWebpage.By = this.browser.constructor.webdriver.By;
    SeleniumWebpage.until = this.browser.constructor.webdriver.until;

    if (readyCondition) {
      if (typeof readyCondition === 'function') {
        await this.driver.wait(readyCondition);
      } else {
        const el = await SeleniumWebpage.until.elementLocated(SeleniumWebpage.By.css(readyCondition));
        await SeleniumWebpage.until.elementIsVisible(el);
      }
    }

    return this.returnProxy()
  }

  runScript(fn, ...args) {
    const parsedFn = __chunk_1.parseFunction(fn, this.getBabelPresetOptions());

    const argStr = parsedFn.args.reduce((acc, v, i) => `${acc}var ${v} = arguments[${i}]; `, '');
    const script = `${argStr}
    ${parsedFn.body}`;

    return this.driver.executeScript(script, ...args)
  }

  runAsyncScript(fn, ...args) {
    const parsedFn = __chunk_1.parseFunction(fn, this.getBabelPresetOptions());

    const argStr = parsedFn.args.reduce((acc, v, i) => `${acc}var ${v} = arguments[${i}]; `, '');
    const script = `${argStr}
    var callback = arguments[arguments.length - 1];
    var retVal = (function() { ${parsedFn.body} })()
    if (retVal && retVal.then) {
      retVal.then(callback)
    } else {
      callback(retVal)
    }
`;
    return this.driver.executeAsyncScript(script, ...args)
  }

  getHtml() {
    return this.driver.getPageSource()
  }

  async getWebElement(selector, fn = el => el) {
    return fn(await this.driver.findElement(SeleniumWebpage.By.css(selector)))
  }

  async getWebElements(selector, fn = els => els) {
    return Promise.all((await this.driver.findElements(SeleniumWebpage.By.css(selector))).map(fn))
  }

  /*
   * WARNING: WebElements seem to return <empty string> instead
   * of null when attribute doesnt exist
   */
  getWebAttribute(selector, attribute) {
    return this.getWebElement(selector, el => el.getAttribute(attribute))
  }

  getWebAttributes(selector, attribute) {
    return this.getWebElements(selector, el => el.getAttribute(attribute))
  }

  async click(selector) {
    const el = await this.getWebElement(selector);
    await el.click();
  }
}

class SeleniumBrowser extends __chunk_2.Browser {
  async _loadDependencies() {
    super._loadDependencies();

    if (SeleniumBrowser.webdriver) {
      return
    }

    SeleniumBrowser.webdriver = await this.loadDependency('selenium-webdriver');
  }

  setHeadless() {
    super.setHeadless();
    this.config.browserArguments.push('headless');
  }

  flushLogs() {}

  async _start(capabilities = {}) {
    const builder = new SeleniumBrowser.webdriver.Builder();

    capabilities = this.getCapabilities(capabilities);
    builder.withCapabilities(capabilities);

    await this.callHook('selenium:build:before', builder);

    this.driver = await builder.build();
  }

  async _close() {
    if (!this.driver) {
      return
    }

    await this.driver.quit();
  }

  _page(url, readyCondition) {
    const page = new SeleniumWebpage(this);
    return page.open(url, readyCondition)
  }
}

exports.default = SeleniumBrowser;
